@@ -26,6 +26,8 @@ import textwrap
 import threading
 import contextlib
 from io import BytesIO, StringIO
+from urllib.parse import urlparse, urlunparse
+from pathlib import PosixPath, PurePath
 from graphviz.backend import ExecutableNotFound
 import pandas as pd
 import requests
@@ -99,6 +101,7 @@ from .types import (
     NodeState,
     NodeStatus,
     NodeTiming,
+    NodeTypeResponse,
     NodeTypes,
     NodeUserColumnsResponse,
     PrettyResponse,
@@ -1895,8 +1898,14 @@ class NodeHandle:
             self,
             key: str,
             chunk: Optional[int],
-            force_refresh: bool = False) -> Optional[ByteResponse]:
-        return self.read_blob(key, chunk, force_refresh).get_content()
+            force_refresh: bool = False,
+            filter_id: bool = True) -> Optional[ByteResponse]:
+        content = self.read_blob(key, chunk, force_refresh).get_content()
+        if filter_id and isinstance(content, pd.DataFrame):
+            content = pd.DataFrame(content.iloc[content["row_id"] >= 0])
+            content = content.set_index("index", drop=True)
+            content.index.name = None
+        return content
 
     def read_all(
             self,
@@ -2050,17 +2059,14 @@ class NodeHandle:
 
     # ModelLike Nodes only
 
-    def get_model_info(self) -> ModelInfo:
-        return cast(ModelInfo, self._client.request_json(
-            METHOD_GET, "/node_model_info", {
-                "dag": self.get_dag().get_uri(),
-                "node": self.get_id(),
-            }))
-
     def is_model(self) -> bool:
         if self._is_model is None:
-            model_info = self.get_model_info()
-            self._is_model = len(model_info) > 0
+            self._is_model = cast(NodeTypeResponse, self._client.request_json(
+                METHOD_GET, "/node_type", {
+                    "dag": self.get_dag().get_uri(),
+                    "node": self.get_id(),
+                }))["is_model"]
+
         return self._is_model
 
     def ensure_is_model(self) -> None:
@@ -2122,6 +2128,7 @@ class BlobHandle:
         self._tmp_uri: Optional[str] = None
         self._owner: Optional[BlobOwner] = None
         self._info: Optional[Dict[str, Any]] = None
+        self._parent: Optional[BlobHandle] = None
 
     def is_full(self) -> bool:
         return self._is_full
@@ -2138,6 +2145,16 @@ class BlobHandle:
         return BlobHandle(
             self._client, f"{self._uri}/{'/'.join(path)}", is_full=True)
 
+    def get_parent(self) -> 'BlobHandle':
+        if self._parent is None:
+            uri = urlparse(self._uri)
+            path = PurePath(*PosixPath(uri.path).parts[:3])
+            new_uri = urlunparse(
+                (uri.scheme, uri.netloc, path.as_posix(), None, None, None))
+            res = BlobHandle(self._client, new_uri, is_full=False)
+            self._parent = res
+        return self._parent
+
     def get_ctype(self) -> Optional[str]:
         return self._ctype
 
@@ -2358,6 +2375,12 @@ class BlobHandle:
                 "reload": reload,
             }))
 
+    def get_model_release(self) -> ModelReleaseResponse:
+        return cast(ModelReleaseResponse, self._client.request_json(
+            METHOD_GET, "/model_release", {
+                "blob": self.get_uri(),
+            }))
+
     def __hash__(self) -> int:
         return hash(self.as_str())
 
@@ -2379,7 +2402,8 @@ class BlobHandle:
 
 
 class CSVBlobHandle(BlobHandle):
-    def finish_csv_upload(self) -> UploadFilesResponse:
+    def finish_csv_upload(
+            self, filename: Optional[str] = None) -> UploadFilesResponse:
         tmp_uri = self._tmp_uri
         assert tmp_uri is not None
         owner = self.get_owner()
@@ -2388,6 +2412,7 @@ class CSVBlobHandle(BlobHandle):
             "csv_uri": self.get_uri(),
             "owner_dag": owner["owner_dag"],
             "owner_node": owner["owner_node"],
+            "filename": filename,
         }
         return cast(UploadFilesResponse, self._client.request_json(
             METHOD_POST, "/finish_csv", args))
@@ -2411,7 +2436,7 @@ class CSVBlobHandle(BlobHandle):
                     fbuff,
                     ext=ext,
                     progress_bar=progress_bar)
-            return self.finish_csv_upload()
+            return self.finish_csv_upload(filename)
         finally:
             self._clear_upload()
 
